@@ -67,6 +67,7 @@ config = {
   'images': None,
   'metadata': None,
   'out_dir': 'output',
+  'max_images': None,
   'use_cache': True,
   'encoding': 'utf8',
   'min_cluster_size': 20,
@@ -142,7 +143,10 @@ def filter_images(**kwargs):
     image_paths.append(i.path)
   # handle the case user provided no metadata
   if not kwargs.get('metadata', False):
-    return [image_paths, []]
+    return [
+      limit_image_count(image_paths, **kwargs),
+      [],
+    ]
   # handle user metadata: retain only records with image and metadata
   l = get_metadata_list(**kwargs)
   meta_bn = set([clean_filename(i.get('filename', '')) for i in l])
@@ -165,7 +169,17 @@ def filter_images(**kwargs):
       metadata.append(d[clean_filename(i)])
   kwargs['metadata'] = metadata
   write_metadata(**kwargs)
-  return [images, metadata]
+  return [
+    limit_image_count(images, **kwargs),
+    limit_image_count(metadata, **kwargs),
+  ]
+
+
+def limit_image_count(arr, **kwargs):
+  '''If the user passed a max_images value, return [:max_images] from arr'''
+  if kwargs.get('max_images', False):
+    return arr[:kwargs['max_images']]
+  return arr
 
 
 def get_image_paths(**kwargs):
@@ -334,7 +348,8 @@ def get_manifest(**kwargs):
     'imagelist': get_path('imagelists', 'imagelist', **kwargs),
     'atlas_dir': kwargs['atlas_dir'],
     'metadata': True if kwargs['metadata'] else False,
-    'centroids': get_centroids(vecs=read_json(layouts['umap']['layout'], **kwargs), **kwargs),
+    'default_hotspots': get_hotspots(vecs=read_json(layouts['umap']['layout'], **kwargs), **kwargs),
+    'custom_hotspots': get_path('hotspots', 'user_hotspots', add_hash=False, **kwargs),
     'config': {
       'sizes': {
         'atlas': kwargs['atlas_size'],
@@ -372,7 +387,7 @@ def get_atlas_data(**kwargs):
   '''
   # if the atlas files already exist, load from cache
   out_dir = os.path.join(kwargs['out_dir'], 'atlases', kwargs['plot_id'])
-  if os.path.exists(out_dir) and kwargs['use_cache']:
+  if os.path.exists(out_dir) and kwargs['use_cache'] and not kwargs.get('shuffle', False):
     print(' * loading saved atlas data')
     return out_dir
   if not os.path.exists(out_dir):
@@ -489,7 +504,7 @@ def get_umap_layout(**kwargs):
     min_dist=kwargs['min_distance'],
     metric=kwargs['metric'])
   # run PCA to reduce dimensionality of image vectors
-  w = PCA(n_components=100).fit_transform(kwargs['vecs'])
+  w = PCA(n_components=min(100, len(kwargs['vecs']))).fit_transform(kwargs['vecs'])
   # fetch categorical labels for images (if provided)
   y = []
   if kwargs.get('metadata', False):
@@ -850,7 +865,7 @@ def read_json(path, **kwargs):
     return json.load(f)
 
 
-def get_centroids(**kwargs):
+def get_hotspots(**kwargs):
   '''Return the stable clusters from the condensed tree of connected components from the density graph'''
   print(' * HDBSCAN clustering data with ' + str(multiprocessing.cpu_count()) + ' cores...')
   config = {
@@ -898,9 +913,9 @@ def get_centroids(**kwargs):
   clusters = sorted(retained, key=lambda i: i['n_images'], reverse=True)
   for idx, i in enumerate(clusters):
     i['label'] = 'Cluster {}'.format(idx+1)
-  # save the centroids to disk and return the path to the saved json
-  print(' * found', len(clusters), 'clusters')
-  return write_json(get_path('centroids', 'centroid', **kwargs), clusters, **kwargs)
+  # save the hotspots to disk and return the path to the saved json
+  print(' * found', len(clusters), 'hotspots')
+  return write_json(get_path('hotspots', 'hotspot', **kwargs), clusters, **kwargs)
 
 
 def get_heightmap(path, label, **kwargs):
@@ -999,6 +1014,7 @@ def parse():
   parser = argparse.ArgumentParser(description=description, formatter_class=argparse.ArgumentDefaultsHelpFormatter)
   parser.add_argument('--images', type=str, default=config['images'], help='path to a glob of images to process', required=False)
   parser.add_argument('--metadata', type=str, default=config['metadata'], help='path to a csv or glob of JSON files with image metadata (see readme for format)', required=False)
+  parser.add_argument('--max_images', type=int, default=config['max_images'], help='maximum number of images to process from the input glob', required=False)
   parser.add_argument('--use_cache', type=bool, default=config['use_cache'], help='given inputs identical to prior inputs, load outputs from cache', required=False)
   parser.add_argument('--encoding', type=str, default=config['encoding'], help='the encoding of input metadata', required=False)
   parser.add_argument('--min_cluster_size', type=int, default=config['min_cluster_size'], help='the minimum number of images in a cluster', required=False)
